/*
**  Module    : support.c
**  Version   : 1.0
**  Date      : 
**  Authors   : Damiano Carra
**
**  Synopsis  : This module contanins all the necessary function able
**              to manage 
**
**  Functions :     void reset (void);
**                  void stop_simulation(void);
**
**  History   : 19-JAN-2006 - Creation of the module.           
**
*/

# ifndef        STREAMING_H
#include "streaming.h"
# endif

/***************************
**  Constant definitions  ** 
***************************/

# ifndef CLOCKS_PER_SEC
#define  CLOCKS_PER_SEC         CLK_TCK
# endif

#define NORMAL_LINE  100  /* identification for lines that contain a      */
                          /* destination traffic fraction                 */
#define COMMENT_LINE 200  /* ...for lines that contain a remark           */
#define ERROR_LINE   400  /* ...for lines that contain no valid data      */
#define BLANCK_LINE  800  /* identification for blanck lines              */

  
/***************************
**  Function Definitions  **
***************************/


/*
**  Function  : int read_input(char *)
**  Return    : number of deltas that composes the distribution
**  Remarks   : it opens input "filename" and reads line by line;  
**
*/
int read_input(char *filename)
{

 FILE *fptr;
 char buffer[255], value_t[50], value_t2[50], value_p[50];
 int i, j, k, read_param, output;
 double cumulative;

///////////////////////////////////////
output = 0;
///////////////////////////////////////
 
   
 if((fptr = fopen(filename, "r")) == NULL)
    {                                           /* no correspondance to an */
     printf("File %s not found\n", filename);   /* existing file: the   */
     exit(1);                                   /* program execution is */ 
    }                                           /* immediately ended.   */
    
//conto quante linee ci sono, ovvero quante delta compongono la pmf
 while(fgets(buffer, 255, fptr) != NULL)
   {
     read_param = sscanf(buffer, "%s %s %s", value_t, value_t2, value_p);
     if(read_param != 3)
        {  printf("File format is not correct\n"); 
           exit(1);
        }
     output++;
   }//while
   
/* output contiene il numero di delta: ora posso crare il vettore base_chunk_vect;
   base_chunk_vect e' una matrice con due righe; quella sotto ha le bande, quella sopra le prob*/
  base_chunk_vect = malloc(3*sizeof(double*));
  for(k = 0; k < 3; k++){
       base_chunk_vect[k] = malloc(output*sizeof(double));
     }//for k     
  // initialize it
  for (i=0; i<3; i++){
     for (j=0; j<output; j++){
          base_chunk_vect[i][j] = 0.0;
         }//for j
     }//for i
      
// faccio il rewind del file
 rewind(fptr);
 output = 0;

// ora lo rileggo memorizzando i dati 
 while(fgets(buffer, 255, fptr) != NULL)
   {
     read_param = sscanf(buffer, "%s %s %s", value_t, value_t2, value_p);
     base_chunk_vect[0][output] = atof(value_p);
     base_chunk_vect[1][output] = atof(value_t);   // download
     base_chunk_vect[2][output] = atof(value_t2);  // upload
     output++;
   } /* while ... */
 
 fclose(fptr);

/* creazione della cumulativa */
 cumulative = 0.0;
 for (i=0; i<output; i++){
     cumulative += base_chunk_vect[0][i];
     base_chunk_vect[0][i] = cumulative;
     }//for i
             
return(output);
}

/*
**  Function  : void shuffle(int dimesione, int *input)
**  Return    : perform a permutation of the element of vector "input"
*/        
void shuffle(int dimesion, Node **input, long *seed){

 int i, rnd_int;
 Node *dummy;
  
 for (i=0; i < dimesion-1; i++){
     rnd_int = (int)uniform(0.0, (double)(dimesion-i), seed);
     dummy = input[i];
     input[i] = input[i+rnd_int]; 
     input[i+rnd_int] = dummy;
 }  
}


/*
**  Function  : void shuffle(int dimesione, int *input)
**  Return    : perform a permutation of the element of vector "input"
*/        
void shuffle_int(int dimesion, int *input, long *seed){

 int i, rnd_int, dummy;
  
 for (i=0; i < dimesion-1; i++){
     rnd_int = (int)uniform(0.0, (double)(dimesion-i), seed);
     dummy = input[i];
     input[i] = input[i+rnd_int]; 
     input[i+rnd_int] = dummy;
 }  
}


/*    
      FUNCTION:       swaps two elements in a vector (of doubles); used in quicksort
      INPUT:          two arrays, positions to be swapped
      OUTPUT:         none
*/
void swap(double *v, int i, int j)
{
  double tmp;

  tmp = v[i];
  v[i] = v[j];
  v[j] = tmp;
}

/*    
      FUNCTION:       swaps two elements in a vector (of int);
      INPUT:          two arrays, positions to be swapped
      OUTPUT:         none
*/
void swap_int(int *v, int i, int j)
{
  int tmp;

  tmp = v[i];
  v[i] = v[j];
  v[j] = tmp;
}

/*    
      FUNCTION:       swaps two elements in a vector (of pointers to Nodes);
      INPUT:          two arrays, positions to be swapped
      OUTPUT:         none
*/
void swap_ptr(Node **v, int i, int j)
{
  Node *tmp;

  tmp = v[i];
  v[i] = v[j];
  v[j] = tmp;
}

/*    
      FUNCTION:       swaps two elements in a vector and a second vector; used in quicksort
      INPUT:          two arrays, positions to be swapped
      OUTPUT:         none
*/
void swap_int_2_vectors(int *v, int i, int j, int *v2)
{
  int tmp;

  tmp = v[i];
  v[i] = v[j];
  v[j] = tmp;
  tmp = v2[i];
  v2[i] = v2[j];
  v2[j] = tmp;
  
}


/*    
      FUNCTION:       implementation of quicksort; second index vector is updated, too
      INPUT:          two arrays, left and right delimiter
      OUTPUT:         none
*/
void sort(double *v, int left, int right)
{
  int k, last;

  if (left >= right) 
    return;
  swap(v, left, (left + right)/2);
  last = left;
  for (k=left+1; k <= right; k++)
    if (v[k] < v[left])
      swap(v, ++last, k);
  swap(v, left, last);
  sort(v, left, last);
  sort(v, last+1, right);
}


/*    
      FUNCTION:       implementation of quicksort (with integers)
      INPUT:          two arrays, left and right delimiter
      OUTPUT:         none
*/
void sort_int(int *v, int left, int right)
{
  int k, last;

  if (left >= right) 
    return;
  swap_int(v, left, (left + right)/2);
  last = left;
  for (k=left+1; k <= right; k++)
    if (v[k] < v[left])
      swap_int(v, ++last, k);
  swap_int(v, left, last);
  sort_int(v, left, last);
  sort_int(v, last+1, right);
}


/*    
      FUNCTION:       implementation of quicksort; second index vector is updated, too
      INPUT:          two arrays, left and right delimiter
      OUTPUT:         none
*/
void sort_int_2_vectors(int *v, int left, int right, int *v2)
{
  int k, last;

  if (left >= right) 
    return;
  swap_int_2_vectors(v, left, (left + right)/2, v2);
  last = left;
  for (k=left+1; k <= right; k++)
    if (v[k] < v[left])
      swap_int_2_vectors(v, ++last, k, v2);
  swap_int_2_vectors(v, left, last, v2);
  sort_int_2_vectors(v, left, last, v2);
  sort_int_2_vectors(v, last+1, right, v2);
}


/*
**  Function  : void reset_stat(void)
**  Return    : none
**  Remarks   : after transient period end, this function is called; 
**              a reset of some performance parameters is done
**
*/
void reset_stat (void)
{
/*probably in the future more variables will be reset*/
}

#define NORMAL_LINE  100  /* identification for lines that contain a      */
                          /* destination traffic fraction                 */
#define COMMENT_LINE 200  /* ...for lines that contain a remark           */
#define ERROR_LINE   400  /* ...for lines that contain no valid data      */
#define BLANCK_LINE  800  /* identification for blanck lines              */

  
/***************************
**  read input  **
***************************/

/*
**  Function  : int parse_line_traffic(char *, char *, char *)
**  Return    : integer: 0 (ok), 1 (not ok) ...
**  Remarks   : internal function used to read the input file, line by line
**
*/
int parse_line_traffic(char *buffer, char *string, char *value)
{
 int read_params, status;
 
 if(buffer == NULL)
        return ERROR_LINE;
 switch(buffer[0])
   {
    case '#' :
         status = COMMENT_LINE;
         break;
         
    case ' ' :
         status = BLANCK_LINE;
         break;

    default:
         read_params = sscanf(buffer, "%s %s", string, value);
         status = (read_params == 2 ? NORMAL_LINE : ERROR_LINE);
         break;
   }
 return status;
} 

/*
**  Function  : Booltype parse_parameters(char *)
**  Return    : TRUE if file is correcly read and parameters set, FALSE otherwise
**  Remarks   : it opens input "filename" and reads line by line; when a parameter
**              is found, it is saved in the structure that will use for simulation  
**
*/
Booltype parse_parameters(char *filename)
{

 FILE *fptr;
 int  num_parameters, line_status;
 char line_buffer[255];
 char name[100], read_value[100];
   
 if((fptr = fopen(filename, "r")) == NULL)
    {                                           /* no correspondance to an */
     printf("File %s not found\n", filename);   /* existing file: the   */
     exit(1);                                   /* program execution is */ 
    }                                           /* immediately ended.   */
    
num_parameters = 0;

 while(fgets(line_buffer, 255, fptr) != NULL)
   {
    line_status = parse_line_traffic(line_buffer, name, read_value);
    switch(line_status)
      {
       case COMMENT_LINE :
         continue;
 
       case BLANCK_LINE :
         continue;
 
       case ERROR_LINE :
         printf("Error in line %5d of file %s\n", num_parameters, filename);
         return FALSE;
 
       case NORMAL_LINE :
         ++num_parameters;
         if (strcmp(name, "attach_to_the_best") == 0)
             {
             attach_to_the_best=atof(read_value);
             continue;
             }
         if (strcmp(name, "let_diffusion") == 0)
             {
             let_diffusion=atoi(read_value);
             continue;
             }
         if (strcmp(name, "threshold_for_fathers") == 0)
             {
             threshold_for_fathers=atoi(read_value);
             continue;
             }
         if (strcmp(name, "prefer_newedge") == 0)
             {
             prefer_newedge=atoi(read_value);
             continue;
             }
         if (strcmp(name, "allow_stripe_change") == 0)
             {
             allow_stripe_change=atoi(read_value);
             continue;
             }
         if (strcmp(name, "drastic_change") == 0)
             {
             drastic_change=atof(read_value);
             continue;
             }             
         break;
 
      } /* switch ... */
   } /* while ... */
 
 fclose(fptr);

/*controls that all parameters are present; at the moment, they are 18*/
   if (num_parameters == 6){
       return(TRUE);
   }
   else {
       return(FALSE);
   }
}



